"""
MetricsFirst SDK for Python
Track analytics for your Telegram bots
"""

from .client import MetricsFirst
from .async_client import AsyncMetricsFirst
from .types import (
    CommandEventData,
    InteractionEventData,
    InteractionType,
    ServiceEventData,
    ErrorEventData,
    ErrorSeverity,
    PurchaseEventData,
    PurchaseStatus,
    RecurringChargeEventData,
    UserIdentifyData,
)

__version__ = "0.1.0"
__all__ = [
    "MetricsFirst",
    "AsyncMetricsFirst",
    "CommandEventData",
    "InteractionEventData",
    "InteractionType",
    "ServiceEventData",
    "ErrorEventData",
    "ErrorSeverity",
    "PurchaseEventData",
    "PurchaseStatus",
    "RecurringChargeEventData",
    "UserIdentifyData",
]




"""
Type definitions for MetricsFirst SDK
"""

from dataclasses import dataclass, field
from enum import Enum
from typing import Optional, Dict, Any


class InteractionType(str, Enum):
    MESSAGE = "message"
    CALLBACK_QUERY = "callback_query"
    INLINE_QUERY = "inline_query"
    COMMAND = "command"
    PHOTO = "photo"
    VIDEO = "video"
    DOCUMENT = "document"
    VOICE = "voice"
    STICKER = "sticker"
    OTHER = "other"


class ErrorSeverity(str, Enum):
    DEBUG = "debug"
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"
    CRITICAL = "critical"


class PurchaseStatus(str, Enum):
    INITIATED = "initiated"
    COMPLETED = "completed"
    FAILED = "failed"
    REFUNDED = "refunded"


@dataclass
class CommandEventData:
    """Data for tracking bot commands"""
    user_id: int
    command: str
    command_args: Optional[str] = None
    response_time_ms: Optional[int] = None
    is_success: bool = True
    error_message: Optional[str] = None


@dataclass
class InteractionEventData:
    """Data for tracking user interactions"""
    user_id: int
    interaction_type: InteractionType = InteractionType.MESSAGE
    content_preview: Optional[str] = None
    callback_data: Optional[str] = None
    response_time_ms: Optional[int] = None


@dataclass
class ServiceEventData:
    """Data for tracking services provided"""
    user_id: int
    service_name: str
    service_type: Optional[str] = None
    is_free: bool = True
    price: float = 0.0
    currency: str = "USD"
    duration_ms: Optional[int] = None
    is_success: bool = True
    error_message: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class ErrorEventData:
    """Data for tracking errors"""
    error_type: str
    error_message: str
    user_id: Optional[int] = None
    error_stack: Optional[str] = None
    error_code: Optional[str] = None
    severity: ErrorSeverity = ErrorSeverity.ERROR
    context: Dict[str, Any] = field(default_factory=dict)


@dataclass
class PurchaseEventData:
    """Data for tracking purchases"""
    user_id: int
    purchase_id: str
    price: float
    status: PurchaseStatus = PurchaseStatus.INITIATED
    tariff_id: Optional[str] = None
    tariff_name: Optional[str] = None
    currency: str = "USD"
    is_recurrent: bool = False
    recurrence_period: Optional[str] = None  # "daily", "weekly", "monthly", "yearly"
    payment_method: Optional[str] = None
    payment_provider: Optional[str] = None
    discount_percent: float = 0.0
    coupon_code: Optional[str] = None
    error_message: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class RecurringChargeEventData:
    """Data for tracking recurring subscription charges"""
    user_id: int
    subscription_id: str
    amount: float
    is_success: bool = True
    currency: str = "USD"
    payment_method: Optional[str] = None
    charge_attempt: int = 1
    error_code: Optional[str] = None
    error_message: Optional[str] = None
    next_retry_at: Optional[str] = None


@dataclass
class UserIdentifyData:
    """Data for identifying users"""
    user_id: int
    username: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    language_code: Optional[str] = None
    is_premium: bool = False
    properties: Dict[str, Any] = field(default_factory=dict)




"""
Synchronous MetricsFirst client
"""

import json
import logging
import threading
import time
from dataclasses import asdict
from queue import Queue, Empty
from typing import Optional, List, Dict, Any
from urllib.request import Request, urlopen
from urllib.error import URLError, HTTPError

from .types import (
    CommandEventData,
    InteractionEventData,
    ServiceEventData,
    ErrorEventData,
    PurchaseEventData,
    PurchaseStatus,
    RecurringChargeEventData,
    UserIdentifyData,
)

logger = logging.getLogger("metricsfirst")


class MetricsFirst:
    """
    Synchronous MetricsFirst SDK client.
    
    Usage:
        mf = MetricsFirst(bot_id="your_bot_id", api_key="your_api_key")
        mf.track_command(CommandEventData(user_id=123, command="/start"))
        mf.shutdown()  # Call on exit to flush remaining events
    """
    
    DEFAULT_API_URL = "https://api.metricsfirst.com"
    
    def __init__(
        self,
        bot_id: str,
        api_key: str,
        api_url: Optional[str] = None,
        batch_events: bool = True,
        batch_size: int = 10,
        batch_interval: float = 5.0,
        debug: bool = False,
        timeout: float = 10.0,
    ):
        """
        Initialize the MetricsFirst client.
        
        Args:
            bot_id: Your bot's unique identifier
            api_key: Your API key from MetricsFirst dashboard
            api_url: Custom API URL (optional)
            batch_events: Whether to batch events before sending
            batch_size: Number of events per batch
            batch_interval: Seconds between batch flushes
            debug: Enable debug logging
            timeout: HTTP request timeout in seconds
        """
        self.bot_id = bot_id
        self.api_key = api_key
        self.api_url = (api_url or self.DEFAULT_API_URL).rstrip("/")
        self.batch_events = batch_events
        self.batch_size = batch_size
        self.batch_interval = batch_interval
        self.timeout = timeout
        
        if debug:
            logging.basicConfig(level=logging.DEBUG)
            logger.setLevel(logging.DEBUG)
        
        self._queue: Queue = Queue()
        self._shutdown = threading.Event()
        self._worker_thread: Optional[threading.Thread] = None
        
        if batch_events:
            self._start_worker()
    
    def _start_worker(self):
        """Start the background worker thread for batching"""
        self._worker_thread = threading.Thread(target=self._worker, daemon=True)
        self._worker_thread.start()
    
    def _worker(self):
        """Background worker that flushes events periodically"""
        events: List[Dict[str, Any]] = []
        last_flush = time.time()
        
        while not self._shutdown.is_set():
            try:
                # Try to get an event with timeout
                event = self._queue.get(timeout=0.1)
                events.append(event)
                
                # Flush if batch is full
                if len(events) >= self.batch_size:
                    self._flush_events(events)
                    events = []
                    last_flush = time.time()
            except Empty:
                pass
            
            # Flush if interval has passed
            if events and (time.time() - last_flush) >= self.batch_interval:
                self._flush_events(events)
                events = []
                last_flush = time.time()
        
        # Flush remaining events on shutdown
        while not self._queue.empty():
            try:
                events.append(self._queue.get_nowait())
            except Empty:
                break
        
        if events:
            self._flush_events(events)
    
    def _flush_events(self, events: List[Dict[str, Any]]):
        """Send a batch of events to the API"""
        if not events:
            return
        
        try:
            for event in events:
                self._send_event(event["type"], event["data"])
        except Exception as e:
            logger.error(f"Failed to flush events: {e}")
    
    def _send_event(self, event_type: str, data: Dict[str, Any]):
        """Send a single event to the API"""
        payload = {
            "botId": self.bot_id,
            "type": event_type,
            "data": data,
        }
        
        try:
            request = Request(
                f"{self.api_url}/api/analytics/track",
                data=json.dumps(payload).encode("utf-8"),
                headers={
                    "Content-Type": "application/json",
                    "X-API-Key": self.api_key,
                },
                method="POST",
            )
            
            with urlopen(request, timeout=self.timeout) as response:
                if response.status != 200:
                    logger.warning(f"API returned status {response.status}")
                else:
                    logger.debug(f"Event sent: {event_type}")
                    
        except HTTPError as e:
            logger.error(f"HTTP error sending event: {e.code} - {e.reason}")
        except URLError as e:
            logger.error(f"URL error sending event: {e.reason}")
        except Exception as e:
            logger.error(f"Error sending event: {e}")
    
    def _track(self, event_type: str, data: Dict[str, Any]):
        """Queue or send an event"""
        if self.batch_events:
            self._queue.put({"type": event_type, "data": data})
        else:
            self._send_event(event_type, data)
    
    def _clean_dict(self, d: Dict[str, Any]) -> Dict[str, Any]:
        """Remove None values and convert enums"""
        result = {}
        for k, v in d.items():
            if v is None:
                continue
            if hasattr(v, "value"):  # Enum
                result[k] = v.value
            elif isinstance(v, dict):
                result[k] = self._clean_dict(v)
            else:
                result[k] = v
        return result
    
    # ==========================================
    # Tracking Methods
    # ==========================================
    
    def track_command(self, data: CommandEventData):
        """Track a bot command"""
        self._track("command", self._clean_dict(asdict(data)))
    
    def track_interaction(self, data: InteractionEventData):
        """Track a user interaction"""
        self._track("interaction", self._clean_dict(asdict(data)))
    
    def track_service(self, data: ServiceEventData):
        """Track a service provided"""
        self._track("service_provided", self._clean_dict(asdict(data)))
    
    def track_error(self, data: ErrorEventData):
        """Track an error"""
        self._track("error", self._clean_dict(asdict(data)))
    
    def track_error_from_exception(
        self,
        exception: Exception,
        user_id: Optional[int] = None,
        severity: str = "error",
        context: Optional[Dict[str, Any]] = None,
    ):
        """Track an error from an exception"""
        import traceback
        
        data = ErrorEventData(
            error_type=type(exception).__name__,
            error_message=str(exception),
            user_id=user_id,
            error_stack=traceback.format_exc(),
            severity=severity,
            context=context or {},
        )
        self.track_error(data)
    
    def track_purchase_initiated(self, data: PurchaseEventData):
        """Track a purchase initiation"""
        data.status = PurchaseStatus.INITIATED
        self._track("purchase_initiated", self._clean_dict(asdict(data)))
    
    def track_purchase_completed(self, data: PurchaseEventData):
        """Track a completed purchase"""
        data.status = PurchaseStatus.COMPLETED
        self._track("purchase_completed", self._clean_dict(asdict(data)))
    
    def track_purchase_error(self, data: PurchaseEventData):
        """Track a failed purchase"""
        data.status = PurchaseStatus.FAILED
        self._track("purchase_error", self._clean_dict(asdict(data)))
    
    def track_recurring_charge_success(self, data: RecurringChargeEventData):
        """Track a successful recurring charge"""
        data.is_success = True
        self._track("recurring_charge_success", self._clean_dict(asdict(data)))
    
    def track_recurring_charge_failed(self, data: RecurringChargeEventData):
        """Track a failed recurring charge"""
        data.is_success = False
        self._track("recurring_charge_failed", self._clean_dict(asdict(data)))
    
    def identify(self, data: UserIdentifyData):
        """Identify a user with properties"""
        payload = self._clean_dict(asdict(data))
        
        try:
            request = Request(
                f"{self.api_url}/api/users/identify",
                data=json.dumps({
                    "botId": self.bot_id,
                    **payload,
                }).encode("utf-8"),
                headers={
                    "Content-Type": "application/json",
                    "X-API-Key": self.api_key,
                },
                method="POST",
            )
            
            with urlopen(request, timeout=self.timeout) as response:
                if response.status != 200:
                    logger.warning(f"Identify returned status {response.status}")
                    
        except Exception as e:
            logger.error(f"Error identifying user: {e}")
    
    def flush(self):
        """Manually flush all pending events"""
        if not self.batch_events:
            return
        
        events = []
        while not self._queue.empty():
            try:
                events.append(self._queue.get_nowait())
            except Empty:
                break
        
        self._flush_events(events)
    
    def shutdown(self):
        """Shutdown the client and flush remaining events"""
        self._shutdown.set()
        
        if self._worker_thread and self._worker_thread.is_alive():
            self._worker_thread.join(timeout=5.0)
        
        logger.debug("MetricsFirst client shutdown complete")
    
    def __enter__(self):
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.shutdown()




"""
Asynchronous MetricsFirst client
"""

import asyncio
import json
import logging
from dataclasses import asdict
from typing import Optional, List, Dict, Any

try:
    import aiohttp
    HAS_AIOHTTP = True
except ImportError:
    HAS_AIOHTTP = False

from .types import (
    CommandEventData,
    InteractionEventData,
    ServiceEventData,
    ErrorEventData,
    PurchaseEventData,
    PurchaseStatus,
    RecurringChargeEventData,
    UserIdentifyData,
)

logger = logging.getLogger("metricsfirst")


class AsyncMetricsFirst:
    """
    Asynchronous MetricsFirst SDK client.
    
    Usage:
        mf = AsyncMetricsFirst(bot_id="your_bot_id", api_key="your_api_key")
        await mf.start()
        await mf.track_command(CommandEventData(user_id=123, command="/start"))
        await mf.shutdown()  # Call on exit to flush remaining events
    """
    
    DEFAULT_API_URL = "https://api.metricsfirst.com"
    
    def __init__(
        self,
        bot_id: str,
        api_key: str,
        api_url: Optional[str] = None,
        batch_events: bool = True,
        batch_size: int = 10,
        batch_interval: float = 5.0,
        debug: bool = False,
        timeout: float = 10.0,
    ):
        """
        Initialize the AsyncMetricsFirst client.
        
        Args:
            bot_id: Your bot's unique identifier
            api_key: Your API key from MetricsFirst dashboard
            api_url: Custom API URL (optional)
            batch_events: Whether to batch events before sending
            batch_size: Number of events per batch
            batch_interval: Seconds between batch flushes
            debug: Enable debug logging
            timeout: HTTP request timeout in seconds
        """
        if not HAS_AIOHTTP:
            raise ImportError(
                "aiohttp is required for AsyncMetricsFirst. "
                "Install with: pip install metricsfirst[async]"
            )
        
        self.bot_id = bot_id
        self.api_key = api_key
        self.api_url = (api_url or self.DEFAULT_API_URL).rstrip("/")
        self.batch_events = batch_events
        self.batch_size = batch_size
        self.batch_interval = batch_interval
        self.timeout = aiohttp.ClientTimeout(total=timeout)
        
        if debug:
            logging.basicConfig(level=logging.DEBUG)
            logger.setLevel(logging.DEBUG)
        
        self._queue: asyncio.Queue = asyncio.Queue()
        self._session: Optional[aiohttp.ClientSession] = None
        self._worker_task: Optional[asyncio.Task] = None
        self._shutdown_event = asyncio.Event()
    
    async def start(self):
        """Start the client and background worker"""
        self._session = aiohttp.ClientSession(timeout=self.timeout)
        self._shutdown_event.clear()
        
        if self.batch_events:
            self._worker_task = asyncio.create_task(self._worker())
        
        logger.debug("AsyncMetricsFirst client started")
    
    async def _worker(self):
        """Background worker that flushes events periodically"""
        events: List[Dict[str, Any]] = []
        
        while not self._shutdown_event.is_set():
            try:
                # Try to get an event with timeout
                event = await asyncio.wait_for(
                    self._queue.get(),
                    timeout=self.batch_interval
                )
                events.append(event)
                
                # Flush if batch is full
                if len(events) >= self.batch_size:
                    await self._flush_events(events)
                    events = []
                    
            except asyncio.TimeoutError:
                # Timeout - flush what we have
                if events:
                    await self._flush_events(events)
                    events = []
            except asyncio.CancelledError:
                break
        
        # Flush remaining events on shutdown
        while not self._queue.empty():
            try:
                events.append(self._queue.get_nowait())
            except asyncio.QueueEmpty:
                break
        
        if events:
            await self._flush_events(events)
    
    async def _flush_events(self, events: List[Dict[str, Any]]):
        """Send a batch of events to the API"""
        if not events:
            return
        
        try:
            tasks = [
                self._send_event(event["type"], event["data"])
                for event in events
            ]
            await asyncio.gather(*tasks, return_exceptions=True)
        except Exception as e:
            logger.error(f"Failed to flush events: {e}")
    
    async def _send_event(self, event_type: str, data: Dict[str, Any]):
        """Send a single event to the API"""
        if not self._session:
            logger.error("Client not started. Call start() first.")
            return
        
        payload = {
            "botId": self.bot_id,
            "type": event_type,
            "data": data,
        }
        
        try:
            async with self._session.post(
                f"{self.api_url}/api/analytics/track",
                json=payload,
                headers={"X-API-Key": self.api_key},
            ) as response:
                if response.status != 200:
                    text = await response.text()
                    logger.warning(f"API returned status {response.status}: {text}")
                else:
                    logger.debug(f"Event sent: {event_type}")
                    
        except aiohttp.ClientError as e:
            logger.error(f"HTTP error sending event: {e}")
        except Exception as e:
            logger.error(f"Error sending event: {e}")
    
    async def _track(self, event_type: str, data: Dict[str, Any]):
        """Queue or send an event"""
        if self.batch_events:
            await self._queue.put({"type": event_type, "data": data})
        else:
            await self._send_event(event_type, data)
    
    def _clean_dict(self, d: Dict[str, Any]) -> Dict[str, Any]:
        """Remove None values and convert enums"""
        result = {}
        for k, v in d.items():
            if v is None:
                continue
            if hasattr(v, "value"):  # Enum
                result[k] = v.value
            elif isinstance(v, dict):
                result[k] = self._clean_dict(v)
            else:
                result[k] = v
        return result
    
    # ==========================================
    # Tracking Methods
    # ==========================================
    
    async def track_command(self, data: CommandEventData):
        """Track a bot command"""
        await self._track("command", self._clean_dict(asdict(data)))
    
    async def track_interaction(self, data: InteractionEventData):
        """Track a user interaction"""
        await self._track("interaction", self._clean_dict(asdict(data)))
    
    async def track_service(self, data: ServiceEventData):
        """Track a service provided"""
        await self._track("service_provided", self._clean_dict(asdict(data)))
    
    async def track_error(self, data: ErrorEventData):
        """Track an error"""
        await self._track("error", self._clean_dict(asdict(data)))
    
    async def track_error_from_exception(
        self,
        exception: Exception,
        user_id: Optional[int] = None,
        severity: str = "error",
        context: Optional[Dict[str, Any]] = None,
    ):
        """Track an error from an exception"""
        import traceback
        
        data = ErrorEventData(
            error_type=type(exception).__name__,
            error_message=str(exception),
            user_id=user_id,
            error_stack=traceback.format_exc(),
            severity=severity,
            context=context or {},
        )
        await self.track_error(data)
    
    async def track_purchase_initiated(self, data: PurchaseEventData):
        """Track a purchase initiation"""
        data.status = PurchaseStatus.INITIATED
        await self._track("purchase_initiated", self._clean_dict(asdict(data)))
    
    async def track_purchase_completed(self, data: PurchaseEventData):
        """Track a completed purchase"""
        data.status = PurchaseStatus.COMPLETED
        await self._track("purchase_completed", self._clean_dict(asdict(data)))
    
    async def track_purchase_error(self, data: PurchaseEventData):
        """Track a failed purchase"""
        data.status = PurchaseStatus.FAILED
        await self._track("purchase_error", self._clean_dict(asdict(data)))
    
    async def track_recurring_charge_success(self, data: RecurringChargeEventData):
        """Track a successful recurring charge"""
        data.is_success = True
        await self._track("recurring_charge_success", self._clean_dict(asdict(data)))
    
    async def track_recurring_charge_failed(self, data: RecurringChargeEventData):
        """Track a failed recurring charge"""
        data.is_success = False
        await self._track("recurring_charge_failed", self._clean_dict(asdict(data)))
    
    async def identify(self, data: UserIdentifyData):
        """Identify a user with properties"""
        if not self._session:
            logger.error("Client not started. Call start() first.")
            return
        
        payload = self._clean_dict(asdict(data))
        
        try:
            async with self._session.post(
                f"{self.api_url}/api/users/identify",
                json={"botId": self.bot_id, **payload},
                headers={"X-API-Key": self.api_key},
            ) as response:
                if response.status != 200:
                    logger.warning(f"Identify returned status {response.status}")
                    
        except Exception as e:
            logger.error(f"Error identifying user: {e}")
    
    async def flush(self):
        """Manually flush all pending events"""
        if not self.batch_events:
            return
        
        events = []
        while not self._queue.empty():
            try:
                events.append(self._queue.get_nowait())
            except asyncio.QueueEmpty:
                break
        
        await self._flush_events(events)
    
    async def shutdown(self):
        """Shutdown the client and flush remaining events"""
        self._shutdown_event.set()
        
        if self._worker_task:
            self._worker_task.cancel()
            try:
                await self._worker_task
            except asyncio.CancelledError:
                pass
        
        # Flush any remaining events
        await self.flush()
        
        if self._session:
            await self._session.close()
        
        logger.debug("AsyncMetricsFirst client shutdown complete")
    
    async def __aenter__(self):
        await self.start()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.shutdown()




"""
Middleware integrations for popular bot frameworks
"""

import time
import functools
from typing import Callable, Optional, Any

from .async_client import AsyncMetricsFirst
from .client import MetricsFirst
from .types import (
    CommandEventData,
    InteractionEventData,
    InteractionType,
    ServiceEventData,
    ErrorEventData,
    ErrorSeverity,
)


# ==========================================
# Decorators
# ==========================================

def track_command(
    client: MetricsFirst,
    get_user_id: Callable[[Any], int],
):
    """
    Decorator to track commands (sync version).
    
    Usage:
        @track_command(mf, lambda ctx: ctx.from_user.id)
        def start_handler(ctx):
            ctx.reply("Hello!")
    """
    def decorator(func: Callable):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            start_time = time.time()
            user_id = get_user_id(args[0] if args else kwargs.get("message") or kwargs.get("ctx"))
            command = f"/{func.__name__}".replace("_handler", "").replace("_command", "")
            
            try:
                result = func(*args, **kwargs)
                response_time = int((time.time() - start_time) * 1000)
                
                client.track_command(CommandEventData(
                    user_id=user_id,
                    command=command,
                    response_time_ms=response_time,
                    is_success=True,
                ))
                
                return result
            except Exception as e:
                response_time = int((time.time() - start_time) * 1000)
                
                client.track_command(CommandEventData(
                    user_id=user_id,
                    command=command,
                    response_time_ms=response_time,
                    is_success=False,
                    error_message=str(e),
                ))
                raise
        
        return wrapper
    return decorator


def async_track_command(
    client: AsyncMetricsFirst,
    get_user_id: Callable[[Any], int],
):
    """
    Decorator to track commands (async version).
    
    Usage:
        @async_track_command(mf, lambda ctx: ctx.from_user.id)
        async def start_handler(ctx):
            await ctx.reply("Hello!")
    """
    def decorator(func: Callable):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            start_time = time.time()
            user_id = get_user_id(args[0] if args else kwargs.get("message") or kwargs.get("ctx"))
            command = f"/{func.__name__}".replace("_handler", "").replace("_command", "")
            
            try:
                result = await func(*args, **kwargs)
                response_time = int((time.time() - start_time) * 1000)
                
                await client.track_command(CommandEventData(
                    user_id=user_id,
                    command=command,
                    response_time_ms=response_time,
                    is_success=True,
                ))
                
                return result
            except Exception as e:
                response_time = int((time.time() - start_time) * 1000)
                
                await client.track_command(CommandEventData(
                    user_id=user_id,
                    command=command,
                    response_time_ms=response_time,
                    is_success=False,
                    error_message=str(e),
                ))
                raise
        
        return wrapper
    return decorator


def track_service(
    client: MetricsFirst,
    service_name: str,
    get_user_id: Callable[[Any], int],
    is_free: bool = True,
    price: float = 0.0,
    currency: str = "USD",
):
    """
    Decorator to track service usage (sync version).
    """
    def decorator(func: Callable):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            start_time = time.time()
            user_id = get_user_id(args[0] if args else kwargs.get("message") or kwargs.get("ctx"))
            
            try:
                result = func(*args, **kwargs)
                duration_ms = int((time.time() - start_time) * 1000)
                
                client.track_service(ServiceEventData(
                    user_id=user_id,
                    service_name=service_name,
                    is_free=is_free,
                    price=price,
                    currency=currency,
                    duration_ms=duration_ms,
                    is_success=True,
                ))
                
                return result
            except Exception as e:
                duration_ms = int((time.time() - start_time) * 1000)
                
                client.track_service(ServiceEventData(
                    user_id=user_id,
                    service_name=service_name,
                    is_free=is_free,
                    price=price,
                    currency=currency,
                    duration_ms=duration_ms,
                    is_success=False,
                    error_message=str(e),
                ))
                raise
        
        return wrapper
    return decorator


def async_track_service(
    client: AsyncMetricsFirst,
    service_name: str,
    get_user_id: Callable[[Any], int],
    is_free: bool = True,
    price: float = 0.0,
    currency: str = "USD",
):
    """
    Decorator to track service usage (async version).
    """
    def decorator(func: Callable):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            start_time = time.time()
            user_id = get_user_id(args[0] if args else kwargs.get("message") or kwargs.get("ctx"))
            
            try:
                result = await func(*args, **kwargs)
                duration_ms = int((time.time() - start_time) * 1000)
                
                await client.track_service(ServiceEventData(
                    user_id=user_id,
                    service_name=service_name,
                    is_free=is_free,
                    price=price,
                    currency=currency,
                    duration_ms=duration_ms,
                    is_success=True,
                ))
                
                return result
            except Exception as e:
                duration_ms = int((time.time() - start_time) * 1000)
                
                await client.track_service(ServiceEventData(
                    user_id=user_id,
                    service_name=service_name,
                    is_free=is_free,
                    price=price,
                    currency=currency,
                    duration_ms=duration_ms,
                    is_success=False,
                    error_message=str(e),
                ))
                raise
        
        return wrapper
    return decorator


def with_error_tracking(
    client: MetricsFirst,
    get_user_id: Callable[[Any], Optional[int]],
    severity: ErrorSeverity = ErrorSeverity.ERROR,
):
    """
    Decorator to automatically track errors (sync version).
    """
    def decorator(func: Callable):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                user_id = None
                try:
                    user_id = get_user_id(args[0] if args else kwargs.get("message") or kwargs.get("ctx"))
                except:
                    pass
                
                client.track_error_from_exception(
                    e,
                    user_id=user_id,
                    severity=severity.value,
                    context={"handler": func.__name__},
                )
                raise
        
        return wrapper
    return decorator


def async_with_error_tracking(
    client: AsyncMetricsFirst,
    get_user_id: Callable[[Any], Optional[int]],
    severity: ErrorSeverity = ErrorSeverity.ERROR,
):
    """
    Decorator to automatically track errors (async version).
    """
    def decorator(func: Callable):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            try:
                return await func(*args, **kwargs)
            except Exception as e:
                user_id = None
                try:
                    user_id = get_user_id(args[0] if args else kwargs.get("message") or kwargs.get("ctx"))
                except:
                    pass
                
                await client.track_error_from_exception(
                    e,
                    user_id=user_id,
                    severity=severity.value,
                    context={"handler": func.__name__},
                )
                raise
        
        return wrapper
    return decorator


# ==========================================
# Aiogram Middleware
# ==========================================

def create_aiogram_middleware(client: AsyncMetricsFirst):
    """
    Create middleware for aiogram 3.x.
    
    Usage:
        from aiogram import Router
        from metricsfirst.middleware import create_aiogram_middleware
        
        router = Router()
        middleware = create_aiogram_middleware(mf)
        router.message.middleware(middleware)
        router.callback_query.middleware(middleware)
    """
    from aiogram import BaseMiddleware
    from aiogram.types import Message, CallbackQuery, Update
    
    class MetricsFirstMiddleware(BaseMiddleware):
        async def __call__(self, handler, event, data):
            start_time = time.time()
            user_id = None
            interaction_type = InteractionType.OTHER
            command = None
            callback_data = None
            
            # Extract user info based on event type
            if isinstance(event, Message):
                user_id = event.from_user.id if event.from_user else None
                
                if event.text and event.text.startswith("/"):
                    interaction_type = InteractionType.COMMAND
                    command = event.text.split()[0]
                elif event.photo:
                    interaction_type = InteractionType.PHOTO
                elif event.video:
                    interaction_type = InteractionType.VIDEO
                elif event.document:
                    interaction_type = InteractionType.DOCUMENT
                elif event.voice:
                    interaction_type = InteractionType.VOICE
                elif event.sticker:
                    interaction_type = InteractionType.STICKER
                else:
                    interaction_type = InteractionType.MESSAGE
                    
            elif isinstance(event, CallbackQuery):
                user_id = event.from_user.id if event.from_user else None
                interaction_type = InteractionType.CALLBACK_QUERY
                callback_data = event.data
            
            try:
                result = await handler(event, data)
                response_time = int((time.time() - start_time) * 1000)
                
                if user_id:
                    if command:
                        await client.track_command(CommandEventData(
                            user_id=user_id,
                            command=command,
                            response_time_ms=response_time,
                            is_success=True,
                        ))
                    else:
                        await client.track_interaction(InteractionEventData(
                            user_id=user_id,
                            interaction_type=interaction_type,
                            callback_data=callback_data,
                            response_time_ms=response_time,
                        ))
                
                return result
                
            except Exception as e:
                response_time = int((time.time() - start_time) * 1000)
                
                if user_id and command:
                    await client.track_command(CommandEventData(
                        user_id=user_id,
                        command=command,
                        response_time_ms=response_time,
                        is_success=False,
                        error_message=str(e),
                    ))
                
                raise
    
    return MetricsFirstMiddleware()

